@@ -34,7 +34,8 @@ native_encode = function(x) {
 #' is_ascii(letters)  # yes
 #' is_ascii(intToUtf8(8212))  # no
 is_ascii = function(x) {
-  out = !is.na(iconv(x, to = 'ascii'))
+  out = iconv(x, to = 'ascii') == x
+  out[is.na(out)] = FALSE
   out[is.na(x)] = NA
   out
 }
