@@ -24,6 +24,8 @@
 #'   arguments of [cache_exec()].
 #' @param print A (typically S3) function that takes the value of an expression
 #'   in the code as input and returns output. The default is [record_print()].
+#'   If a non-function value (e.g., `NA`) is passed to this argument, [print()]
+#'   (or [show()] for S4 objects) will be used.
 #' @param print.args A list of arguments for the `print` function. By default,
 #'   the whole list is not passed directly to the function, but only an element
 #'   in the list with a name identical to the first class name of the returned
@@ -240,10 +242,14 @@ record = function(
     # print value (via record_print()) if visible
     if (!is_error(out) && out$visible) {
       if (is.null(print)) print = record_print
+      if (!is.function(print)) print = record_print.default
       p_args = print.args
+      val = out$value
       # index print args by first class of out unless args are wrapped in I()
-      if (!inherits(p_args, 'AsIs')) p_args = p_args[[class(out$value)[1]]]
-      out = handle_eval(do.call(print, c(list(out$value), p_args)))
+      if (!inherits(p_args, 'AsIs')) p_args = p_args[[class(val)[1]]]
+      out = handle_eval(if (length(p_args) == 0) print(val) else do.call(
+        print, c(list(val), p_args), quote = inherits(val, c('name', 'call'))
+      ))
       if (length(out) && !is_error(out)) {
         if (is.list(out)) lapply(out, add_result) else add_result(out)
       }
@@ -321,13 +327,17 @@ new_record = function(x, class) structure(x, class = paste0('record_', class))
 .record_cls = c('source', 'output', 'message', 'warning', 'error', 'plot', 'asis')
 .record_classes = paste0('record_', .record_cls)
 
+# default device arguments
+dev_args = list(units = 'in', onefile = FALSE, width = 8, height = 8, res = 84)
+
 dev_open = function(dev, file, args) {
   m = names(formals(dev))
-  a = list(units = 'in', onefile = FALSE, width = 8, height = 8, res = 84)
-  for (i in names(a)) {
-    if (i %in% m && is.null(args[[i]])) args[[i]] = a[[i]]
+  for (i in names(dev_args)) {
+    if (i %in% m && is.null(args[[i]])) args[[i]] = dev_args[[i]]
   }
+  n = length(dev.list())
   do.call(dev, c(list(file), args))
+  if (length(dev.list()) <= n) stop('Failed to open device for recording plots.')
   dev.control('enable')
   dev.cur()
 }
@@ -341,7 +351,7 @@ dev_ext = function(dev) {
 
 is_error = function(x) inherits(x, 'try-error')
 
-#' @param to The output format (text or html).
+#' @param to The output format (text, markdown, or html).
 #' @param encode For HTML output, whether to base64 encode plots.
 #' @param template For HTML output, whether to embed the formatted results in an
 #'   HTML template. Alternatively, this argument can take a file path, i.e.,
@@ -353,12 +363,33 @@ is_error = function(x) inherits(x, 'try-error')
 #' @return The `format()` method returns a character vector of plain-text output
 #'   or HTML code for displaying the results.
 format.xfun_record_results = function(
-    x, to = c('text', 'html'), encode = FALSE, template = FALSE, ...
+    x, to = c('text', 'markdown', 'html'), encode = FALSE, template = FALSE, ...
 ) {
-  if (to[1] == 'text') {
+  alt = 'A plot recorded by xfun::record()'
+  if (to[1] != 'html') {
+    is_md = to[1] == 'markdown'
     res = unlist(lapply(x, function(z) {
-      if (!inherits(z, c('record_source', 'record_asis'))) z = paste('#>', z)
-      gsub('\n*$', '\n', one_string(z))
+      if (length(z) == 0) return()
+      cls_all = sub('^record_', '', class(z)); cls = cls_all[1]
+      if (cls != 'asis') {
+        if (is_md && cls == 'plot') {
+          z = sprintf('![%s](<%s>)', alt, z)
+        } else {
+          z = gsub('^(\\s*\n)+|\n\\s*$', '', one_string(z))  # trim blank lines
+          if (cls != 'source') z = paste('#>', split_lines(z))
+          if (is_md) {
+            cls_all = if (any(c('message', 'warning', 'error') %in% cls_all)) {
+              c('plain', cls_all)
+            } else if (cls == 'source') {
+              replace(cls_all, cls_all == 'source', 'r')
+            } else setdiff(cls_all, 'output')
+            z = fenced_block(z, sprintf('.%s', cls_all))
+          }
+        }
+      }
+      z = one_string(z)
+      if (!is_md) z = gsub('\n*$', '\n', z)
+      z
     }))
     return(raw_string(res))
   }
@@ -366,8 +397,8 @@ format.xfun_record_results = function(
     cls = sub('^record_', '', class(z))
     if (cls == 'asis') z else if (cls == 'plot') {
       sprintf(
-        '<p class="%s"><img src="%s" alt="A plot recorded by xfun::record()" /></p>',
-        cls, if (encode) vapply(z, base64_uri, '') else URLencode(z)
+        '<p class="%s"><img src="%s" alt="%s" /></p>',
+        cls, if (encode) vapply(z, base64_uri, '') else URLencode(z), alt
       )
     } else {
       paste0(
@@ -375,7 +406,7 @@ format.xfun_record_results = function(
           '<pre class="%s"><code>',
           if (cls == 'source') paste0('language-r" data-start="', attr(z, 'lines')[1]) else cls
         ),
-        escape_html(one_string(z)), '</code></pre>'
+        html_escape(one_string(z)), '</code></pre>'
       )
     }
   }))
@@ -392,7 +423,7 @@ make_title = function(x) {
   if (length(x) == 0) return('')
   x = gsub('^#+\\s*', '', x[1])  # remove possible comment chars
   x = gsub('\\s*[.]*$', '... | ', x)  # add ... to the end
-  escape_html(x)
+  html_escape(x)
 }
 
 #' @param x An object returned by `record()`.
